#include "common.h"
/*
calcpar.c; calculates optical parameters and scores input SPD (spectral power distribution)
(c)2023,2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/
#define VS_CALCSPDPAR 240619
#define MIN_VIABLE_CCT 100
struct SPDparameters calcSPDpar(struct SPD s,struct LightConfig e, struct Goal g){  
	struct Coord uv;
	struct SPDparameters p={0};
	int j;
	if(FLG_ERR2SCREEN)fprintf(stderr,"\ncalcSPDpar(): '%s' ...",s.fname);
	p=calcCIE1931(s,ciebar, p); 
	p.cct=calcCCT(p); 
	if(p.cct<MIN_VIABLE_CCT){p.score=REJECT;return(p);} 
	switch(g.cct){
		case LIMITMAX: if(p.cct>g.p.cct){p.score=REJECT;return(p);} break;
		case LIMITMIN: if(p.cct<g.p.cct){p.score=REJECT;return(p);} break;
		case MAXIMIZE: break;
		case MINIMIZE: break;
		case DONTCARE: break;
	}	
	uv=calcUV(p);
	p.duv[DUV_ACTUAL]=calcDUV(uv); 
	p.duv[DUV_TARGET]=calcDUVtarget(p.cct); 
	p.duv[DUV_DELTA]=p.duv[DUV_ACTUAL]-p.duv[DUV_TARGET];	
	switch(g.duv){
		case LIMITMAX: if(fabs(p.duv[DUV_DELTA])>fabs(g.p.duv[DUV_DELTA])){p.score=REJECT;return(p);} break;
		case LIMITMIN: break;
		case MAXIMIZE: break;
		case MINIMIZE: p.score-=(fabs(p.duv[DUV_DELTA])*10000.0); break;
		case DONTCARE: break;
	}			
	p=calcCRI(s,ciebar,cri_s,tcs,p);
	for(j=0;j<2;j++)
		switch(g.cri[j]){
			case LIMITMAX: break;
			case LIMITMIN: if(p.cri[j]<g.p.cri[j]){p.score=REJECT;return(p);} break;
			case MAXIMIZE: p.score+=(p.cri[j]*10.0); break;
			case MINIMIZE: break;
			case DONTCARE: break;
		}		
	p=calcAlphaOpicELR(s,action,p);
	p.a_opic[_PLUX]*=e.scaleFac; // normalize to room 
	p.mEDI*=e.scaleFac* e.etaG;  // normalize to room + switch to vertical plane
	for(j=0;j<6;j++)
		switch(g.a_opic[j]){
			case LIMITMIN: if(p.a_opic[j]<g.p.a_opic[j]){p.score=REJECT;return(p);} break;
			case LIMITMAX: if(p.a_opic[j]>g.p.a_opic[j]){p.score=REJECT;return(p);} break;
			case MAXIMIZE: p.score+=p.a_opic[j]; break;
			case MINIMIZE: p.score-=p.a_opic[j]; break;
			case DONTCARE: break;
		}			
	switch(g.mEDI){
		case LIMITMIN: if(p.mEDI<g.p.mEDI){p.score=REJECT;return(p);} break;
		case LIMITMAX: if(p.mEDI>g.p.mEDI){p.score=REJECT;return(p);} break;		
		case MAXIMIZE: p.score+=p.mEDI/10.0;break;
		case MINIMIZE: break;
		case DONTCARE: break;
	}
	for(j=0;j<2;j++)p.cost[j]=s.cost[j];
	for(j=0;j<2;j++)
		switch(g.cost[j]){
			case LIMITMIN: break;
			case LIMITMAX: if(p.cost[j]>g.p.cost[j]){p.score=REJECT;return(p);} break;		
			case MAXIMIZE: break;
			case MINIMIZE: p.score-=p.cost[0];break;
			case DONTCARE: break;
		}
			
	p.mEDIperW = p.mEDI/p.cost[POWER];
	switch(g.mEDIperW){
		case LIMITMAX: break;
		case LIMITMIN: if(p.mEDIperW<g.mEDIperW){p.score=REJECT;return(p);} break;
		case MAXIMIZE: p.score+=p.mEDIperW;break;
		case MINIMIZE: break;
		case DONTCARE: break;
	}
	p.lxperW = p.a_opic[_PLUX]/p.cost[POWER];
	switch(g.lxperW){
		case LIMITMAX: break;
		case LIMITMIN: if(p.lxperW<g.lxperW){p.score=REJECT;return(p);} break;
		case MAXIMIZE: p.score+=p.lxperW;break;
		case MINIMIZE: break;
		case DONTCARE: break;
	}
	p.irrad=calcIrrad(s)*e.scaleFac;
	if(FLG_ERR2SCREEN)fprintf(stderr,"\ncalcSPDpar(): '%s' scored as...%lf",s.fname,p.score);	
	return(p);
}	