#include "common.h"
#define VS_CALIBRATE 230913

/*
calibrate.c; calibrates a lab measured spectrum based on a NIST standard and its accompanying lab measurement
(c)2023 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

char *dirout="out";
#define LINEWIDTH 5	
void gnuplotCMD(char *mode,char *fnameNoExt, char *fnameSTD){
	FILE *f;
	char fname[MFNAME];
	sprintf(fname,"%s.plt",fnameNoExt);
	if((f=fopen(fname,"w"))==NULL)goto errOpen;	
	fprintf(f,"set title 'Calibrate Spectrum %s using %s' font \",16\"\n",fnameNoExt,mode);
	fprintf(f,"set datafile separator ','\n");
	fprintf(f,"set xlabel 'Wavelength [nm]' font \",16\"\n");
	fprintf(f,"set xrange [400:750] \n");
	fprintf(f,"set xtics 25 \n");
	fprintf(f,"set yrange [0:*] \n");
	fprintf(f,"set y2range [0:*] \n");
	fprintf(f,"set ytics  \n");
	fprintf(f,"set grid \n");
	fprintf(f,"set key left top \n");
	fprintf(f,"set ylabel 'Raw Counts/Channel' font \",16\"  \n");
	fprintf(f,"plot \"%s.txt\" using 1:2 skip 1 with lines lw %d title 'measured(relative)' \n",fnameNoExt,LINEWIDTH);	
	fprintf(f,"set y2label 'Energy/s/nm' font \",16\" \n");		
	fprintf(f,"set y2range [0:*] \n");	
	fprintf(f,"set y2tics  \n");
	fprintf(f,"replot \"%s//%s.csv\" using 1:2  with lines lw %d axis x1y2 title 'calibrated' \n",dirout,fnameNoExt,LINEWIDTH);
	if(strlen(fnameSTD)>0)fprintf(f,"replot \"in//%s.csv\" using 1:2  with lines lw %d axis x1y2 title 'standardNIST'  \n",fnameSTD,LINEWIDTH);		
	fprintf(f,"pause mouse\n");		
	fclose(f);
	fprintf(stderr,"\ngnuplotCMD(): plt file written."); 
	fprintf(stderr,"\n\t- system 'gnuplot %s'",fname);	
	return;
errOpen: fprintf(stderr,"\ngnuplotCMD() ERR can't open '%s'",fname);exit(0);	
}
struct SPD calibrate(char *fnameTST, char *mode, char *fnameSTD){
	struct SPD s,gt,exp;
	int nm;
	fprintf(stderr,"calibrate(%d) %s ",VS_CALIBRATE,fnameTST);
	sprintf(s.fname,"%s.txt",fnameTST);
	s=readInterpSPD(s.fname);
	switch(mode[0]){
		case 's': fprintf(stderr,"\n\t-using %s as NIST standard (s[] *= gt[]/exp[])",gt.fname);	
			sprintf(gt.fname,"in/%s",fnameSTD);
			sprintf(exp.fname,"%s.txt",fnameSTD);
			exp=readInterpSPD(exp.fname);
			gt=readSPD(gt.fname);		
			for(nm=NM_START;nm<NM_END;nm++)s.y[nm]*=gt.y[nm]/exp.y[nm];		
			break;
		case 'c': fprintf(stderr,"\n\t-using counts2energy (counts/time_bin/wavelength_bin - (*hc/lambda) -> Energy/time_bin/wavelength_bin)");
			for(nm=NM_START;nm<NM_END;nm++)s.y[nm]*=(H*C/nm);
			break;
		case 'n': fprintf(stderr,"\n\tMODE: no calibration (do nothing)"); break;
		default: goto errDev;
	}
	strcpy(s.fname,fnameTST);
	return(s);
errDev: fprintf(stderr,"ERR undefined [s(cale by std) || c(ounts) || n(o scale)]");exit(3);
}
int main(int argc, char *argv[]){	
	struct SPD s;
	if(argc<3)goto errArg;
	if(argv[2][0]=='s' && argc!=4)goto errArg;
	s=calibrate(argv[1],argv[2],argv[3]);
	writeSPD(s,1,2,dirout);
	fprintf(stderr,"\n\t-writing to %s/%s.csv",dirout,argv[1]);
	gnuplotCMD(argv[2],argv[1],argv[3]);	
	return(0);
errArg:fprintf(stderr,"calibrate()\nUSAGE: 'calibrate fname(no ext) mode ...' i.e."
	"\n\t'calibrate fname(no ext) standard（NIST calibrated) fnameSTD'"
	"\n\t'calibrate fname(no ext) counts2energy\n"); exit(0);
}

