#include "common.h"
#define CCT_5000K 5000
#define CRIX 8
#define CRI_MESSAGE 0

/*
cri.c; calculates CRI Ra and Re for a given SPD (spectral power distribution)
(c)2023 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/




struct SPD calcRef_BlackBody(double cct,struct SPD ciebar[]){
	int nm;
	double sum=0.0;
	struct SPD pl, ref;
	for(nm=NM_START;nm<NM_END;nm++){
    	pl.y[nm] = (3.7415E-16)/((pow((nm*NM2M),5))*((exp(0.014388/(nm*NM2M*cct)))-1));
    	sum+=pl.y[nm]*ciebar[Y].y[nm];
    }
    for(nm=NM_START;nm<NM_END;nm++)ref.y[nm]=(pl.y[nm]/sum)*100;
    if(CRI_MESSAGE)fprintf(stderr,"\n\tcalcRef_BlackBody(%.0f K) (CCT<%dK)",cct,CCT_5000K);
    return(ref);
}	
struct SPD calcRef_StandardIlluminateD(double cct, struct SPD cri_s[]){
	int nm;
	struct SPD  Sa, ref;
	double xD,yD,M1,M2;
	double denom;
    if(cct<7000)xD = ((-4.6070*1000000000)/(pow(cct,3))+((2.9678*1000000)/(pow(cct,2)))+(99.11/cct)+0.244063);
    else xD = ((-2.0064*1000000000)/(pow(cct,3))+((1.9018*1000000)/(pow(cct,2)))+(247.48/cct)+0.237040);

    yD = (-3 * xD * xD) + (2.870 * xD) - 0.275;
    denom = 0.0241 + 0.2562 * xD - (0.7341 * yD);
    M1 = (-1.3515 - (1.7703 * xD) + (5.9114 * yD))/ denom;
    M2 = (0.03 - 31.4424 * xD + 30.0717 * yD)/ denom;
    for(nm=NM_START;nm<NM_END;nm++){Sa.y[nm]= cri_s[X].y[nm] + M1*cri_s[Y].y[nm] + M2 *cri_s[Z].y[nm];}
    for(nm=NM_START;nm<NM_END;nm++)ref.y[nm]= (Sa.y[nm] * 100)/ Sa.y[200]; /*??????????what is this fixed number???*/
    if(CRI_MESSAGE)fprintf(stderr,"\n\tcalcRef_StandardIlluminateD(%.0f) CCT>%dK",cct,CCT_5000K);
    return(ref);
}

struct Coord calcUV_TCS(double x, double y){
	struct Coord uv;
	double denom=(12.0*y -2.0*x + 3.0);
	uv.x=4.0*x/denom;
	uv.y=6.0*y/denom;
	return(uv);
}
struct Coord calcCD(struct Coord uv){
	struct Coord cd;
	cd.x = (4.0 - uv.x - 10.0*uv.y)/uv.y;
	cd.y = (1.708*uv.y + 0.404 - 1.481*uv.x)/uv.y;
	return(cd);
}   
struct Coord calcUVkibar(struct Coord cdk, struct Coord cdr, struct Coord cdki){
	struct Coord uvkibar, fac;
	double denom;
	fac.x=cdr.x/cdk.x*cdki.x;
	fac.y=cdr.y/cdk.y*cdki.y;
    denom = 16.518 + 1.481*fac.x - fac.y;
	uvkibar.x = (10.872 + 0.404*fac.x - 4.0*fac.y)/denom;
	uvkibar.y = 5.520/denom;
	return(uvkibar);
}

struct Coord3 calcUVW(double y, struct Coord uv, struct Coord uvkibar){
	struct Coord3 uvw;
	uvw.w = 25.0 * (pow(y,(1.0/3.0))) - 17.0;
	uvw.u = 13.0 * uvw.w * (uvkibar.x - uv.x);
	uvw.v = 13.0 * uvw.w * (uvkibar.y - uv.y);
//	fprintf(stderr,"\ncalcUVW(): %.4lf %.4lf %.4lf -u-> %.4lf", uvw.w,uvkibar.x,uv.x,uvw.u);	
	return(uvw);
}
double calcR(struct Coord3 uvw[]){
    double delE = sqrt(pow((uvw[1].w-uvw[0].w),2.0)+pow((uvw[1].u-uvw[0].u),2.0)+pow((uvw[1].v-uvw[0].v),2.0));
    double r = 100.0 - (4.6 * delE);
//    fprintf(stderr,"\ncalcR():  delEuvw=%.2lf --> R=%6.2lf",delE,r);
    return(r);
}

#define VS_CALCCRI 230628
struct SPDparameters calcCRI(struct SPD spec, struct SPD ciebar[], 
	struct SPD cri_s[], struct SPD tcs[], struct SPDparameters par_s){
	int nm,k,j,i;
	double tmp=0.0;
	struct SPD spd[2];
	double sumprod[2][3];
	struct SPDparameters p;
	struct Coord uv[4],cd[4], uvkibar[4];
	struct Coord3 uvw[4];
	double r[MTCS];	
	par_s.cri[0]=0.0; par_s.cri[1]=0.0;
	if(fabs(par_s.duv[DUV_ACTUAL])>DUV_LIMIT_CRI)goto errDUV;	
    j=0; // test
    strcpy(spd[j].fname,spec.fname); 
    for(nm=NM_START;nm<NM_END;nm++)tmp+=spec.y[nm]*ciebar[Y].y[nm];
    for(nm=NM_START;nm<NM_END;nm++)spd[j].y[nm]=spec.y[nm]/tmp*100.0;
    sprintf(spd[j].fname,"tst      "); 
	p=calcCIE1931(spd[j], ciebar, p); 
	uv[j]=calcUV(p);     
	cd[j]=calcCD(uv[j]);   

	j=1;	// ref.
	if(p.cct < CCT_5000K){
		spd[j]=calcRef_BlackBody(par_s.cct,ciebar); 
		sprintf(spd[j].fname,"bb  %.0fK",par_s.cct);		
	}
	else{
		spd[j]=calcRef_StandardIlluminateD(par_s.cct, cri_s);
		sprintf(spd[j].fname,"IlD %.0fK",par_s.cct);		
	}	
	sprintf(spd[j].fname,"ref %.0fK",par_s.cct);
	p=calcCIE1931(spd[j], ciebar, p);   
	uv[j]=calcUV(p);   			        
	cd[j]=calcCD(uv[j]); 
	
	for(j=0;j<2;j++)uvkibar[j]=calcUVkibar(cd[0],cd[1],cd[j]);
	if(CRI_MESSAGE){	
		nm=480; 
		fprintf(stderr,"\ncalcCRI(): spec.y[%d]=%.2e spd[0].y[%d]=%.3f spd[1].y[%d]=%.3f",nm,spec.y[nm],nm,spd[0].y[nm],nm,spd[1].y[nm]);
		for(j=0;j<2;j++)fprintf(stderr,"\ncalcCRI(): uv[%d]=(%7.4f %7.4f) cd[%d]=(%7.4f %7.4f)",
			j,uv[j].x,uv[j].y,j,cd[j].x,cd[j].y);
	}
	
    for(k=1;k<MTCS;k++){ // make 2 for test
    	for(j=0;j<2;j++){
    		for(i=0;i<3;i++){
    			sumprod[j][i]=0.0;
    			for(nm=NM_START;nm<NM_END;nm++)sumprod[j][i]+=tcs[k].y[nm]*spd[j].y[nm]*ciebar[i].y[nm];
    		}
    		tmp=sumprod[j][0]+sumprod[j][1]+sumprod[j][2];
    		sumprod[j][2]=sumprod[j][1]; // save Y in the Z spot as Z is no longer needed
    		for(i=0;i<2;i++)sumprod[j][i]/=tmp;
    		uv[j+2]=calcUV_TCS(sumprod[j][0],sumprod[j][1]);
    		cd[j+2]=calcCD(uv[j+2]);
    		uvkibar[j+2]=calcUVkibar(cd[0],cd[1],cd[j+2]);
    		if(j==0)uvw[j]=calcUVW(sumprod[j][2],uvkibar[j],uvkibar[j+2]);
    		else uvw[j]=calcUVW(sumprod[j][2],uv[j],uv[j+2]);
    	}
    	r[k] = calcR(uvw);	
    	if(CRI_MESSAGE){ // set K to positive value if with to print!
    		fprintf(stderr,"\n\ncalcCRI(): %02d...",k);
    		fprintf(stderr,"\n\tsumprod-%02d: T %.4lf %.4lf %.4lf R %.4lf %.4lf %.4lf ",
    			k,sumprod[0][0],sumprod[0][1],sumprod[0][2],sumprod[1][0],sumprod[1][1],sumprod[1][2]);
    		fprintf(stderr,"\n\tuv-%02d     : T %.4lf %.4lf R %.4lf %.4lf",k,uv[2].x,uv[2].y,uv[3].x,uv[3].y);
    		fprintf(stderr,"\n\tcd-%02d     : T %.4lf %.4lf R %.4lf %.4lf",k,cd[2].x,cd[2].y,cd[3].x,cd[3].y);
    		fprintf(stderr,"\n\tuvkibar   : T %.4lf %.4lf R %.4lf %.4lf",uvkibar[2].x,uvkibar[2].y,uvkibar[3].x,uvkibar[3].y);
    		fprintf(stderr,"\n\tuvw       : T %.4lf %.4lf %.4lf R %.4lf %.4lf  %.4lf",uvw[0].w,uvw[0].u,uvw[0].v,uvw[1].w,uvw[1].u,uvw[1].v);
    		fprintf(stderr,"\n\tR(%02d)     : %.2lf",k,r[k]);
    	}
	}
	
	for(k=1;k<=CRIX; k++)par_s.cri[0]+=r[k];
	par_s.cri[0]/=CRIX;
	for(k=1;k<MTCS;k++)par_s.cri[1]+=r[k]; 
	par_s.cri[1]/=(MTCS-1);
//	fprintf(stderr,"\ncalcCRI(%d) : R8=%.0f R14=%.0f ",VS_CALCCRI,par_s.cri[0],par_s.cri[1]);	
	return(par_s);
errDUV: 
//	fprintf(stderr,"\ncalcCRI(%d) duv=%.4lf > %.4lf -> NO CRI ",VS_CALCCRI,par_s.duv,DUV_LIMIT);
	return(par_s);
}