#include "common.h"
#define VS_DAY2ESP 241002
#define FULL 65535

/*
day2esp.c; converts an 24-hour dynamic lighting schedule to one implementable on ESP32 board
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/


void writeSch2Esp(int minterval,struct SPD_weight wVector[], char *head){
	int iline, ich;
	char bb[MB];
	FILE *f;
	fprintf(stderr,"\nwriteSch2Esp(%d) --> 'espsched.h' ",VS_DAY2ESP);
	if((f=fopen("espsched.h","w"))==NULL)goto errOpen;
	fprintf(f,"#define VS_SCHED %ld\n",atol(bb+2));
	fprintf(f,"#define TIME_ON_S  25200 //  07:00 \n#define TIME_OFF_S 82800 //  23:00\n");
	fprintf(f,"#define MINTERVAL %d\n",minterval);
	fprintf(f,"//%s",head);
	fprintf(f,"const struct LightStatus espsched[MINTERVAL]={\n");
	for(iline=1;iline<minterval;iline++){
		fprintf(f,"{%3d,%5s,{",iline,wVector[iline].fname);
		for(ich=0;ich<MCH-1;ich++)fprintf(f,"%5d,",(int) (wVector[iline].fraction[ich]*FULL));
		fprintf(f,"%5d}}", (int) wVector[iline].fraction[MCH-1]*FULL);
		if(iline!=(minterval-1))fprintf(f,",\n");
		else fprintf(f,"\n};\n");
	}
	fprintf(f,"const struct LightStatus schBinary=\n");
	iline=7;
	fprintf(f,"{%3d,%5s,{",iline,wVector[iline].fname);
	for(ich=0;ich<MCH-1;ich++)fprintf(f,"%5d,",(int) (wVector[iline].fraction[ich]*FULL));
	fprintf(f,"%5d}};",(int) (wVector[iline].fraction[MCH-1]*FULL));
	fclose(f);
	return;
errOpen: fprintf(stderr," ERR Can't open 'espsched.h' \n"); exit(0);
}	
void day2esp(char *fname){
	FILE *fweight=fopen(fname,"r");
	struct SPD_weight wVector[MINTERVAL];
	char head[MB];
	int minterval=0;
	fprintf(stderr,"day2esp(%d) %s --> ",VS_DAY2ESP, fname);
	if(fweight==NULL)goto errOpen;
	fgets(head,MB,fweight);
	while(minterval<MINTERVAL){
		minterval++;
		wVector[minterval]=readWeight(fweight);
		if(strlen(wVector[minterval].fname)<1)break;
	}
	fclose(fweight);	
	writeSch2Esp(minterval,wVector,head);
	return;
errOpen: fprintf(stderr," ERR Can't open '%s'",fname);exit(1);	
}
int main(int argc, char *argv[]){
	if(argc<2)goto errArg;
	day2esp(argv[1]);
	return(0);
errArg: fprintf(stderr,"USAGE: day2esp fname.csv");	
	return(1);
}