#include "common.h"
#define VS_DAYINTERP 230906
#define FULL 65535
#define SIMU_INC_SEC 100

/*
dayinterp.c; based on SPDs at certain times of the day, interpolate SPDs between setpoints. 
(c)2023 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/




void simuSchedule(int minterval,struct SPD_weight  wVector[], char *head){
	int s,k=0, klow=0,kup=0, ich;
	struct SPD_weight  w;
	FILE *f;
	if((f=fopen("dayinterp.csv","w"))==NULL)goto errOpen;
	fprintf(f,"%s",head);	
	for(s=0;s<1440*60;s+=SIMU_INC_SEC){
		fprintf(f,"%6d,",s); 
		k=0;				
		while(s>atol(wVector[k].fname))k++;
		if(k==0)for(ich=0;ich<MCH;ich++){fprintf(f,"%5.3lf,",wVector[0].fraction[ich]); continue;}
		klow=k-1;
		kup=k;		
		for(ich=0;ich<MCH;ich++){
			w.fraction[ich]=wVector[klow].fraction[ich]+
				((s-atol(wVector[klow].fname)) * (wVector[kup].fraction[ich]-wVector[klow].fraction[ich])) 
				/ (atol(wVector[kup].fname) - atol(wVector[klow].fname));
		}
		for(ich=0;ich<MCH;ich++)fprintf(f,"%5.3lf,",w.fraction[ich]);
		fprintf(f,"\n"); 		
	}	
	return;
errOpen: fprintf(stderr," ERR Can't open 'dayinterp.csv' for output \n");exit(1);
}

void dayinterp(char *fname){
	FILE *fweight=fopen(fname,"r");
	struct SPD_weight wVector[MINTERVAL];
	char head[MB];
	int minterval=0;
	fprintf(stderr,"dayinterp(%d) <-- %s --> dayinterp.csv\n",VS_DAYINTERP, fname);
	if(fweight==NULL)goto errOpen;
	fgets(head,MB,fweight);
	while(minterval<MINTERVAL){
		wVector[minterval]=readWeight(fweight);
		if(strlen(wVector[minterval].fname)<1)break;
		minterval++;
	}
	fclose(fweight);	
	simuSchedule(minterval,wVector,head);
	return;
errOpen: fprintf(stderr," ERR Can't open '%s' for input",fname);exit(1);	
}
int main(int argc, char *argv[]){
	if(argc<2)goto errArg;
	dayinterp(argv[1]);
	return(0);
errArg: fprintf(stderr,"USAGE:  'dayinterp fname.csv'");	
	return(1);
}