#include "common.h"
#define VS_GENPERM 240520
#define TEST_GETWEIGHTPERM 0
#define UNIQUE 1
#define REPEAT 0
#define PERM_MAXPWM 100
/*
getWeightPerm.c; calculates alpha-optic for a given SPD (spectral power distribution)
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/



void prnCombo(int nperm,int *pwm,int *mpwm,int nch);
int chkUnique(int *pwm,int nch,int step); /* check if ratio is unique or already processed (repeat) */
void maxPower(int *pwm, int nch, int *mpwm); /* max pwm for given ratio */
struct SPD_weight genPerm(struct SPD_weight w,int nch);

int chkUnique(int *pwm,int nch,int step){
	int ich,ind=0;
	for(ich=0;ich<nch;ich++)if(pwm[ich]<pwm[ind] && pwm[ich]!=0)ind=ich;
	if(pwm[ind]==step)return(UNIQUE);
	if(pwm[ind]==0)return(0);
	ich=0;
	while(ich<nch && (pwm[ich]%pwm[ind])==0)ich++;
	if(ich==nch)return(REPEAT);
	return(UNIQUE);
}
void maxPower(int *pwm, int nch, int *mpwm){
	int ich,indmax=0,convfactor; 
	for(ich=1;ich<nch;ich++)if(pwm[ich]>pwm[indmax])indmax=ich;
	convfactor=PERM_MAXPWM/pwm[indmax];
	for(ich=0;ich<nch;ich++)mpwm[ich]=pwm[ich]*convfactor;
}
struct SPD_weight genPerm(struct SPD_weight w,int nch){
	static int pwm[MCH]={0,0,0},mpwm[MCH], maxpwm[MCH];
	static unsigned long int nperm=0;
	static int firstCall=1;
	static int istep=1;
	int ich;
	
	if(firstCall){
		firstCall=0;
		fprintf(stdout,"getWeight(): Maximum PWM is set at 100\n");
		fprintf(stdout,"getWeight(): Integer Step Size [1 100)>");
		fscanf(stdin,"%d",&istep);
		if(PERM_MAXPWM%istep)goto error;
		for(ich=0;ich<nch;ich++){
			fprintf(stdout,"Max PWM for ch%d>",ich);
			fscanf(stdin,"%d",&maxpwm[ich]);
		}			
				
		fprintf(stdout,"getPerm(): Generating all unique PWM combinations"
			"with step interval %d for\n",istep); 
		for(ich=0;ich<nch;ich++)fprintf(stdout,"\tch[%d]=[0 %d]\n",ich,maxpwm[ich]);
	}	
	do{
		ich=0;
		while(pwm[ich]>=maxpwm[ich]){
			pwm[ich]=0;
			ich++; 
		}
		if(ich==nch)goto finish;
		pwm[ich]+=istep;
	}while(!chkUnique(pwm,nch,istep));
	maxPower(pwm,nch,mpwm);
	nperm++;	
	prnCombo(nperm,pwm,mpwm,nch);
	for(ich=0;ich<nch;ich++)w.fraction[ich]= 0.01 * (double) mpwm[ich];	
	return(w);
finish:
	fprintf(stderr,"\ngenPerm()\n\tchannels: %d\n\tstep: %d"
		"\n\tTotal : %e\n\tUnique : %ld\n\tPercent: %lf%%",
		nch,istep,pow((double) nch,(double)((100.0)/(double)istep)),nperm,
		(double) nperm * 100.0/ pow((double) nch,(double)(100.0)/(double)istep) );
	for(ich=0;ich<nch;ich++)w.fraction[ich]= -1.00;	
	return(w);
error: fprintf(stderr,"genPerm(%d) ERR The maximum PWM must be an integer multiple of step size.",VS_GENPERM); exit(0);		
	
}	
#if TEST_GETWEIGHTPERM ==1
int main(void){
	int nch=3,step=20;
	struct SPD_weight w={"test1",{0,0}};
	w=genPerm(w,nch);
	while(w.fraction[0]>-0.5){
		fprintf(stdout,"\t-->\t");
    	for(int ich=0;ich<nch;ich++)fprintf(stdout,"%.3lf:",w.fraction[ich]);
		w=genPerm(w,nch);		
	}
	return(0);
}
#endif
void prnCombo(int nperm,int *pwm,int *mpwm,int nch){
#if TEST_GETWEIGHTPERM==1
	int ich;
	static int flgFirst=1;
	if(flgFirst){
		flgFirst=0;
		fprintf(stdout,"perm\tratio\t\tratio(max)\tw.fraction[j]");        
	}	
	fprintf(stdout,"\n%4d\t",nperm);
	for(ich=0;ich<nch;ich++)fprintf(stdout,"%d:",pwm[ich]);
	fprintf(stdout,"\t-->\t");
	for(ich=0;ich<nch;ich++)fprintf(stdout,"%d:",mpwm[ich]);	
#endif	
}
#if TEST_GETWEIGHTPERM==2
int main(void){
	int step=5, nch=4, maxpwm=100;
	struct SPD_weight w={"test2",{0,0}};
	w=genPerm(w,nch);
	while(w.fraction[0]>-0.5)w=genPerm(w,nch);
	return(0);
}
#endif	
	
	
	
