#include "common.h"
#define VS_GETWEIGHT 240516
#define TEST 0
/*
getweight.c; subroutines return the PWM settings to be used to build an SPD combining the SPDs of individual LEDs
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

struct SPD_weight genPerm(struct SPD_weight w,int nch); /* in file: getWeightperm.c */
struct SPD_weight readWeight(FILE *fweight);
struct SPD_weight genWeightBruteForce(struct SPD_weight w,int nch, double step); /* ansari brute force modify */

struct SPD_weight getWeight(FILE *fweight,struct SPD_weight w,int nch, char *mode){
	static int firstCall=1;
	static double step=0.05;
	switch(mode[0]){
		case 'r':
			if(firstCall)fprintf(stderr,"getWeight(): read from file\n");
			w = readWeight(fweight);break;
		case 'b':
			if(firstCall){
				fprintf(stdout,"getWeight(): brute force mode. Input step size(0  1) -- suggest 0.05>");
				fscanf(stdin,"%lf",&step);
				fprintf(stdout,"genWeightBruteForce(): Step size set at %.3lf",step);
			}
			w = genWeightBruteForce(w,nch,step);break;
		case 'p': w = genPerm(w,nch);break;			
		default: goto errMode;
	}
	firstCall=0;
	return(w);	
errMode: fprintf(stderr,"\ngetWeight(): ERR mode=%s in not valid: Choose from [read brute perm]",mode); exit(1);

}


struct SPD_weight readWeight(FILE *fweight){
	int ich;
	struct SPD_weight w={"\0",{-1.0,0} ,"\0"};	
	if(fgets(bb,MB,fweight)==NULL){fclose(fweight);return(w);}
	if(strlen(bb)<10)goto eof_data;
	strcpy(w.fname,strtok(bb,","));	
	for(ich=0;ich<MCH;ich++)w.fraction[ich]=atof(strtok(NULL,","));
	return(w);
eof_data: fprintf(stderr,"\n readSPDweight() ERR unexpected end-of-line '%s'",bb);
	exit(1);	
}


struct SPD_weight genWeightBruteForce(struct SPD_weight w,int nch,double step){
	static double mpwm[MCH] = {1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
	int j,k;
	if(w.fraction[0]<(mpwm[0]+EPS))w.fraction[0]+=step;
	for(j=0;j<nch-1;j++)
		if (w.fraction[0]>(mpwm[0]+EPS)){for(k=0;k<j+1;k++)w.fraction[j]=0.0;w.fraction[j+1]+=step;}
	if (w.fraction[nch-1]>(mpwm[nch-1]+EPS)){strcpy(w.fname,"\0");return(w);}
	return(w);
}

#if TEST==1
int main(void){
	struct SPD_weight w={"test",{0,0,0,0} ,"\0"};	
	struct SPDparameters p;
	FILE *fweight=fopen("weight.csv","r");
	char mode[8]="b";
	int ich;
	fprintf(stderr,"TEST %c\n",mode[0]);
	while(1){
		w=getWeight(fweight,w,4,mode);
		fprintf(stdout,"\n'%s',",w.fname);		
		for(ich=0;ich<MCH;ich++)fprintf(stdout,"%.2lf,",w.fraction[ich]);
		if(strlen(w.fname)<1)break;
	}
	return(0);
}
#endif