#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define VS_GNUPLOTSPD 230902
#define MFNAME 128
#define PLOTS_PER_PAGE 5
#define LINEWIDTH 5	

/*
gnuplotSPD.c; subroutines support whitespd in writing a file to plt SPD for gnuplot
(c)2023 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/


int gnuplotSPDopen(char *fname, char *title,char *ylabel){
	FILE *f=fopen(fname,"w");
	if(f==NULL)goto errOpen;	
	fprintf(f,"set title '%s' font \",16\"\n",title);
	fprintf(f,"set datafile separator ','\n");
	fprintf(f,"set key samplen 6 spacing 1.6 font \",16\"\n");
	fprintf(f,"set xlabel 'Wavelength [nm]' font \",16\"\n");
	fprintf(f,"set xrange [400:750]\nset xtics 25 font \",14\"\n");
	fprintf(f,"set ylabel '%s' font \",16\" \n",ylabel);
	fprintf(f,"set yrange [0:*]    \nset ytics font \",14\" \n");
	fprintf(f,"set grid \n");
	fclose(f);
	return(0);
errOpen: fprintf(stderr,"\ngnuplotSPDopen() ERR Open '%s'",fname);exit(1);
}
int gnuplotSPD(char *dir, char *fname_data, char *fname_plt, char type){
	FILE *f=fopen(fname_plt,"a");
	static int iplt=0;
	if(f==NULL)goto errOpen;
	fprintf(stderr,"\ngnuplotSPD(%d) - adding '%s//%s to '%s'",
		VS_GNUPLOTSPD,dir,fname_data,fname_plt);
	if((iplt%PLOTS_PER_PAGE)>0)fprintf(f,"re");
	if(type=='-')fprintf(f,"plot \"%s//%s.csv\" with lines lw %d\n",
		dir,fname_data,LINEWIDTH);
	else fprintf(f,"plot \"%s//%s_%c.csv\" with lines lw %d\n",
		dir,fname_data,type,LINEWIDTH);
	iplt++;	
	if(iplt%PLOTS_PER_PAGE==0)fprintf(f,"pause mouse\n");
	fclose(f);
	return(iplt);
errOpen: fprintf(stderr,"\ngnuplotSPD() ERR Open '%s'",fname_plt);exit(1);
}
int gnuplotSPDclose(char *fname,int iplt){
	FILE *f;
	if((iplt%PLOTS_PER_PAGE)>0){
		if((f=fopen(fname,"a"))==NULL)goto errOpen;	
		fprintf(f,"pause mouse\n");	
		fclose(f);
	}
	fprintf(stderr,"\ngnuplotSPDclose():\n\t- plt file closed");
	fprintf(stderr,"\n\t- system 'gnuplot %s'",fname);
	
	return(0);
errOpen: fprintf(stderr,"\ngnuplotSPDclose() ERR Open '%s'",fname);exit(1);
}
#define MLINE 2056
void plotSPD(char *fname){
	FILE *f=fopen(fname,"r");
	int iplt;
	char line[MLINE];
	if(f==NULL)goto errOpen;
	fgets(line,MLINE,f);
	gnuplotSPDopen("combine.plt","Radiant Flux","uW/nm");	
	while((fgets(line,MLINE,f))!=NULL)
		iplt=gnuplotSPD("out",strtok(line,","),"combine.plt",'-');	
	gnuplotSPDclose("combine.plt",iplt);
	return;
errOpen: fprintf(stderr,"\nplotSPD ERR Open '%s'",fname);exit(1);	
}