#include "common.h"
#define VS_POST 241030
#define MBIN 2000 
#define BIG_NUMBER 1.0e24
#define UPDATE 1
#define DO_NOTHING 0
/*
post.c; cpost processing of brute force spectrum to create tables
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

int flg_update(struct SPDparameters new,struct SPDparameters old, struct Goal g){
	int k;
	// Reject if not the best SPD to date for condition
	if(g.lxperW==MAXIMIZE)if(new.score<old.score)return(DO_NOTHING);
	if(g.mEDIperW==MAXIMIZE)if(new.mEDIperW<old.mEDIperW)return(DO_NOTHING);	
	if(g.cct==MAXIMIZE)if(new.cct<old.cct)return(DO_NOTHING);	
	if(g.cct==MINIMIZE)if(new.cct>old.cct)return(DO_NOTHING);	
	for(k=0;k<MALPHA;k++)if(g.a_opic[k]==MAXIMIZE)
		if(new.a_opic[k]<old.a_opic[k])return(DO_NOTHING);
	for(k=0;k<MALPHA;k++)if(g.a_opic[k]==MINIMIZE)
		if(new.a_opic[k]<old.a_opic[k])return(DO_NOTHING);	
	// Reject SPD that fail inclusion criteria
	if(g.duv==LIMITMAX)
		if(fabs(new.duv[DUV_DELTA])>g.p.duv[DUV_DELTA])return(DO_NOTHING);
	for(k=0;k<2;k++)if(g.cri[k]==LIMITMIN)
		if(new.cri[k]<g.p.cri[k])return(DO_NOTHING);
	return(UPDATE);
}

int goalMinMax(struct Goal g){
	int k;
	if(g.duv==MAXIMIZE || g.duv==MINIMIZE)return(g.duv);
	if(g.cct==MAXIMIZE || g.cct==MINIMIZE)return(g.cct);
	for(k=0;k<2;k++)if(abs(g.cri[k])==MAXIMIZE)return(g.cri[k]);
	for(k=0;k<MALPHA;k++)if(abs(g.a_opic[k])==MAXIMIZE)return(g.a_opic[k]);	
	if(abs(g.mEDI)==MAXIMIZE)return(g.mEDI);
	if(abs(g.irrad)==MAXIMIZE)return(g.irrad);	
	for(k=0;k<2;k++)if(abs(g.cost[k])==MAXIMIZE)return(g.cost[k]);
	for(k=0;k<2;k++)if(abs(g.user[k])==MAXIMIZE)return(g.user[k]);
	if(abs(g.mEDIperW)==MAXIMIZE)return(g.mEDIperW);
	if(abs(g.lxperW)==MAXIMIZE)return(g.lxperW);	
	return(1);
}
void prnUsage(){
	fprintf(stdout,"USAGE: post size_of_bin number_of_channels_to_read x-axis < inputFileName > outputFileName\n"
		"\tinputFileName: name of file with SPD parameters calculated by whitespd\n"
		"\tParameter to maximize/minimize (1 only) and limits (many as like) are set in 'goal.csv'\n"
		"\tX-axis: 'm' OR 'cct' \n"
		"\te.g. maximum(m-EDI)=f(cct) subject to inclusion criteria in goal.csv of CRI>90, Duv<0.001\n"		
		"\tOUTPUT: a file formatted  'x-axis bind,maximized/minimized quantity'\n" );	
}

int main(int argc, char *argv[]){ 
	int k=0,kmin,kmax, nch, xaxis;
	double binsize;
	char headerLine[MB];
	unsigned long long int j=0; 
	struct SPD_weight iw={0},ow[MBIN]={{0,0}};
	struct SPDparameters ip={0},op[MBIN]={{0,0}};
	struct Goal goal;
	int binCount[MBIN]={0};
	if(argc!=4)goto errArg;
	binsize=atof(argv[1]);
	nch=atol(argv[2]);
	xaxis=argv[3][0];	
	goal=readGoal();

	for(k=0;k<MBIN;k++)op[k].score=goalMinMax(goal)*BIG_NUMBER; 
	fgets(headerLine,MB,stdin);headerLine[strlen(headerLine)-1]='\0';fprintf(stdout,"%s",headerLine);
	while(ip.score>(REJECT+1)){
		j++; if(j%1000000==0)fprintf(stderr,"\r%llu",j);
		ip=readSPDpar(stdin,ip,&iw,nch);
		switch(xaxis){
			case 'm': k= (int) ip.a_opic[_M]/binsize; break;
			case 'c': k= (int) ip.cct/binsize; break;
			default: goto errBin;
		}
		if(k>(MBIN-1)|| k<0)goto errRange;
		binCount[k]++;
		if(flg_update(ip,op[k],goal)){op[k]=ip; ow[k]=iw;} 
	}
	k=0;while(binCount[k]==0)k++;kmin=k;
	k=MBIN-1;while(binCount[k]==0)k--;kmax=k;
	for(k=kmin-1;k<kmax+1;k++){
		if(binCount[k]==0)op[k].score=0.0;
		writeSPDpar(stdout,k*binsize + binsize/2,"-",op[k],ow[k],nch);
	}
	return(0);
errRange:fprintf(stderr,"\nERR index (%d): line %llu",k,j);exit(1);
errArg: prnUsage(); exit(2);
errBin: prnUsage(); exit(3);
}

	

