#include "common.h"
#define VS_OCEAN 230913

/*
procOcean.c; converts an SPD measured by the ocean optics spectrometer to form usable by whitespd
(c)2023 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

int main(int argc, char *argv[]){
	FILE *f,*ftmp;
	char fname[MFNAME];
	int tc_ms, n;
	double d[3];

	if(argc!=2)goto errArg;

	fprintf(stderr,"procOcean(%d)\n",VS_OCEAN); 
	sprintf(fname,"%s.inf",argv[1]);
	if((f=fopen(fname,"r"))==NULL)goto errOpen;	
	fgets(bb,MB,f); // header for 3 col data
	tc_ms=atol(strtok(bb,"\t"));
	n=atol(strtok(NULL,"\t"));
	fclose(f);
	fprintf(stderr,"\t- reading %s: tc = %d ms naverage = %d (saved, not used)\n",fname,tc_ms,n);

	sprintf(fname,"%s.dat",argv[1]);
	if((f=fopen(fname,"r"))==NULL)goto errOpen;	
	fgets(bb,MB,f); 
	fprintf(stderr,"\t- reading %s (3 col TAB separated):   'pixel TAB nm TAB counts/time_bin/wavelength_bin'\n",fname);
	sprintf(fname,"%s.txt",argv[1]);
	if((ftmp=fopen(fname,"w"))==NULL)goto errOpen;	

	fprintf(ftmp,"%d,%d\n",tc_ms,n);
	fprintf(stderr,"\t- writing %s (2 col COMMA separated): line 1: 'time constant, naverage'",fname);
	fprintf(stderr," then 'nm,counts/%d [ms]/wavelength_bin'\n",tc_ms);
	while((fgets(bb,MB,f))!=NULL){
		d[0]=atof(strtok(bb,"\t"));
		d[1]=atof(strtok(NULL,"\t"));
		d[2]=atof(strtok(NULL,"\t"));
		fprintf(ftmp,"%.3lf,%.3lf\n",d[1],d[2]);
	}
	fclose(f);
	fclose(ftmp);
	return(0);
errArg:	fprintf(stderr,"USAGE procOcean fname(no ext)\n");exit(0);
errOpen:
	fprintf(stderr,"ERR");  exit(0);
}

