#include "common.h"
/*
read.c; subroutines support whitespd in readingoptical constant arrays, and input LED spectra
(c)2023,2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

#define VS_READSPD 231125
struct SPD readSPD(char *fnameSPD){
	FILE *f;
	struct SPD s = {"filename",{0}, {10,18},{"default"},1000};
	int nm,iline=0;
	double tmp;
	char fname[64];
	sprintf(fname,"%s.csv",fnameSPD);
	strcpy(s.fname,fnameSPD);
	if((f=fopen(fname,"r"))==NULL)goto errOpen;
	while(fscanf(f,"%d,%lf",&nm,&tmp)==2){s.y[nm]=tmp;iline++;}
	fclose(f);
	sprintf(bprn,"\n\treadSPD(%d)  fr '%s' (nm,J/nm):  1nm step) %d lines",
		VS_READSPD,fname,iline);
	sprintf(fname,"%s_d.csv",fnameSPD);
	if((f=fopen(fname,"r"))==NULL){
		sprintf(bprn,"%s, cost/power set to DEFAULT. ('%s' doesn't exist)",bprn,fname); 				
	}else{
		fgets(bb,MB,f); s.cost[X]=atof(bb); 
		fgets(bb,MB,f); s.cost[Y]=atof(bb); 
		fgets(bb,MB,f); strcpy(s.stripInfo,bb);
		fclose(f);
		sprintf(bprn,"%s; fr '%s' cost, power, info",bprn,fname); 
	}
	prnlog(bprn,PRNLOG);
	return(s);
errOpen: fprintf(stderr,"\nreadSPD(%d) ERR '%s' doesn't exist.****\n",VS_READSPD,fname);exit(2);
}

#define VS_readSPD_Interp2col 230903
struct SPD readInterpSPD(char *fname){
	FILE *f;
	struct SPD s;
	struct SPD_point d[2]={{0.0,0.0}};
	int nm; /*wavelength [nm] */
	int n; /* number of times spectra averaged (Not Used)*/
	int tc_ms; /* time constant at which data taken [ms] (set at 1000) */
	double m,b; /* line from 2 points: y = mx + b */
	strcpy(s.fname,fname);
	if(FLG_ERR2SCREEN)fprintf(stderr,"\n\treadInterpSPD(%d)",VS_readSPD_Interp2col);
	if((f=fopen(fname,"r"))==NULL)goto err_Open;
	fscanf(f,"%d,%d",&tc_ms,&n);

	fscanf(f,"%lf,%lf",&d[1].nm,&d[1].y);
	if(d[1].nm>NM_START)goto err_missingBlue;
	for(nm=NM_START;nm<NM_END;nm++){
	    while (d[1].nm<nm){
	    	d[0]=d[1]; 
	    	if(fscanf(f,"%lf,%lf",&d[1].nm,&d[1].y)!=2)goto err_missingData;
	    } 
	    m =(d[1].y - d[0].y)/(d[1].nm - d[0].nm);
	    b = d[1].y - m*d[1].nm; 	    
	    s.y[nm] = (m * nm + b)* (1000.0/tc_ms)/(d[1].nm-d[0].nm);
	}
	if(FLG_ERR2SCREEN)fprintf(stderr," '%s' J/%d[ms]/%.1lf[nm]--> J/s/nm ", 
		fname,tc_ms,d[1].nm-d[0].nm);		
	fclose(f);
	return(s);
err_Open: fprintf(stderr,"readInterpSPD( ) ERR file '%s' does not exist.\n",fname);exit(1);
err_missingBlue:fprintf(stderr,"readInterpSPD( )ERR starts %lf [nm] > %d\n",d[1].nm,NM_START); exit(2); 	
err_missingData:fprintf(stderr,"readInterpSPD() ERR ends. %lf [nm] (nm<%d)\n",d[0].nm,nm); exit(3); 
}

#define VS_READEXPPARAM 230628
struct LightConfig readLightConfig(void){
	struct LightConfig e={0.626,991,289113,'a',"default"};
//	struct LightConfig e={0.56,1,1,'b',"calibration"};
	FILE *f=fopen("lightconfig.csv","r"); 
	if(f==NULL)fprintf(stderr,"\nreadLightConfig() defaults " );
	else{
		fgets(bb,MB,f); e.etaG=atof(bb);
		fgets(bb,MB,f); e.refEh=atof(bb);
		fgets(bb,MB,f); e.simEh=atof(bb);		
		fgets(bb,MB,f); e.s=bb[0];
		fgets(e.spd,MB,f); 
		fclose(f);
		fprintf(stderr,"\nreadLightConfig() fr 'lightconfig.csv' ");
	}
	e.scaleFac=e.refEh/e.simEh;
	fprintf(stderr,"etaG=%.3lf refEh=%.0lf simEh=%.2e -> scale=%.4lf ('%c %s')",
		e.etaG,e.refEh,e.simEh,e.scaleFac,e.s,e.spd);
	return(e);
}
#define VS_READGOAL 241015
struct Goalline {int goal;double threshold;};
struct Goalline readGoalLine(FILE *f){
	struct Goalline gline;
	char *ptr;
	if(fgets(bb,MB,f)==NULL)goto err;
	ptr=strtok(bb,",");  if(ptr==NULL)goto err; gline.goal=atol(ptr);
	ptr=strtok(NULL,",");if(ptr==NULL)goto err; gline.threshold=atof(ptr);
	ptr=strtok(NULL,",");if(ptr==NULL)goto err;
	switch(gline.goal){
		case LIMITMAX: fprintf(stderr,"\n\t%s <= %lf",ptr,gline.threshold);break;
		case LIMITMIN: fprintf(stderr,"\n\t%s >= %lf",ptr,gline.threshold);break;
		case MAXIMIZE: fprintf(stderr,"\n \t%s maximize",ptr,gline.threshold);break;
		case MINIMIZE: fprintf(stderr,"\n\t%s minimize",ptr,gline.threshold);break;		
		default: break;
	}		
	return(gline);
err:fprintf(stderr,"\nreadGoalLine(%d) ERR data format incorrect",VS_READGOAL);exit(1);		
}
struct Goal readGoal(void){
	int j;
	struct Goal g={0};
	struct Goalline gline;
	FILE *f=fopen("goal.csv","r");
	fprintf(stderr,"\nreadGoal(%d)",VS_READGOAL);
	if(f==NULL){fprintf(stderr," optimization file 'goal.csv' NOT found. not optimizing");return(g);}
	fprintf(stderr," fr 'goal.csv'");
	gline=readGoalLine(f); g.duv=gline.goal; g.p.duv[DUV_DELTA]=gline.threshold; 
	gline=readGoalLine(f); g.cct=gline.goal; g.p.cct=gline.threshold; 	
	for(j=0;j<2;j++){gline=readGoalLine(f); g.cri[j]=gline.goal; g.p.cri[j]=gline.threshold;}	
	for(j=0;j<6;j++){gline=readGoalLine(f); g.a_opic[j]=gline.goal; g.p.a_opic[j]=gline.threshold;}	
	gline=readGoalLine(f); g.mEDI=gline.goal; g.p.mEDI=gline.threshold; 
	gline=readGoalLine(f); g.irrad=gline.goal; g.p.irrad=gline.threshold; 	
	for(j=0;j<2;j++){gline=readGoalLine(f); g.cost[j]=gline.goal; g.p.cost[j]=gline.threshold;}		
	gline=readGoalLine(f); g.mEDIperW=gline.goal; g.p.mEDIperW=gline.threshold;
	gline=readGoalLine(f); g.lxperW=gline.goal; g.p.lxperW=gline.threshold;
	fclose(f);
	return(g);	
}

#define VS_READOPTICALCONSTANTS 230902
void readOpticalConstants(void){
	int ich;
	char fname[64];
	fprintf(stderr,"\nreadOpticalConstants(%d): CIE1931 files...",VS_READOPTICALCONSTANTS);	
	ciebar[X]=readInterpSPD("cie1931/xbar.csv");	
	ciebar[Y]=readInterpSPD("cie1931/ybar.csv");
	ciebar[Z]=readInterpSPD("cie1931/zbar.csv");	
	fprintf(stderr,"\nreadOpticalConstants(%d): alpha opic files...",VS_READOPTICALCONSTANTS);
	action[0]=readInterpSPD("alpha/scone.csv");	
	action[1]=readInterpSPD("alpha/mcone.csv");	
	action[2]=readInterpSPD("alpha/lcone.csv");	
	action[3]=readInterpSPD("alpha/rhodopic.csv");	
	action[4]=readInterpSPD("alpha/melanopic.csv");	
	action[5]=readInterpSPD("alpha/vlamda.csv");		
	fprintf(stderr,"\nreadOpticalConstants(%d): CRI Ra Rf files...",VS_READOPTICALCONSTANTS);	
	for(ich=0;ich<3;ich++){
		sprintf(fname,"cri/S%d.csv",ich);
		cri_s[ich]=readInterpSPD(fname);
	}
	for(ich=1;ich<MTCS;ich++){
		sprintf(fname,"cri/tcs%02d.csv",ich);
		tcs[ich]=readInterpSPD(fname);;
	}
}