#include "common.h"
/*
tradparam.c; calculates Duv, CCT, CRI, CIE, irradience
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

struct Coord calcUV(struct SPDparameters p){
	struct Coord uv;
	double d=((-2.0*p.cie[X]) + (12.0*p.cie[Y])+3.0*p.ciesum);
	uv.x = (4.0*p.cie[X]) / d;
	uv.y = (6.0*p.cie[Y]) / d;
	return(uv);
}
double calcDUV(struct Coord uv){
	double lfp, a, lbb=0.0,duv;
	double k[7]={-0.471106,1.925865,-2.4243787, 1.5317403,-0.5179722, 0.0893944, -0.00616793};
	int j;
	lfp = sqrt(pow(uv.x-0.292,2)+ pow(uv.y-0.24,2));
	a = acos(((uv.x-0.292)/lfp));
	for(j=0;j<7;j++)lbb+=pow(a,j)*k[j];
	duv = lfp - lbb;
	return(duv);
}
double calcDUVtarget(double t){
	if(t<2870)return(0);
	else return(57700/t/t - 44.6/t + 0.00854);
}
double calcCCT(struct SPDparameters p){
    double tmp,cct;
    if(fabs((p.cie[Y]/p.ciesum)-0.1858)<EPS)goto warnEPS;
    tmp=((p.cie[X]/p.ciesum)-0.3320)/(0.1858-(p.cie[Y]/p.ciesum));
    cct = 437.0 * pow(tmp,3) + 3601.0 * pow(tmp,2) + 6861.0 * tmp + 5517.0;
    return(cct);
warnEPS: if(FLG_ERR2SCREEN)fprintf(stderr,"\ncalcCCT(): |CIEy| < 0.1858 (CCT not calc)"); 
	return(0);
}
struct SPDparameters calcCIE1931(struct SPD s, struct SPD bar[], struct SPDparameters p){
	int nm,k; 
	p.ciesum=0.0;
	for(k=0;k<3;k++){
		p.cie[k]=0.0;
		for(nm=NM_START;nm<NM_END;nm++)p.cie[k]+= bar[k].y[nm] * s.y[nm];	
	}
	for(k=0;k<3;k++)p.ciesum+=p.cie[k];
	for(k=0;k<2;k++)p.ciexy[k]=p.cie[k]/p.ciesum;
	return(p);		
}

double calcIrrad(struct SPD s){
	int nm; double irrad=0.0;
	for(nm=NM_START;nm<NM_END;nm++)irrad+=s.y[nm];
	return(irrad);
}
