#include "common.h"
#define VS_WRITE 220916

/*
write.c; subroutines support whitespd in writing SPDs and their calculated parameters
(c)2023, 2024 Jonathon David White (jonathondavid@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see 
<https://www.gnu.org/licenses/gpl-3.0.en.html>.

If you choose to make use of this program or subroutines in your work, 
we would appreciate it if you would cite our paper 
"Building Circadian Effective Spectra: A C Language Toolkit"

You may download the latest version of the code from https://www.whiteslight.com
*/

int writeSPD(struct SPD s,int nm_step, int flgCol, char *dirout){ /* write spectra w/ nm steps*/
	FILE *f;
	int nm;
	if(flgCol>=3){fprintf(stderr,"writeSPD() not available");return(0);}
	sprintf(bb,"%s/%s.csv",dirout,s.fname);
	if((f=fopen(bb,"w"))==NULL)goto errOpen;
	for(nm=NM_START;nm<NM_END;nm+=nm_step){
		if(flgCol==2)fprintf(f,"%d,",nm);
		fprintf(f,"%.2e\n",s.y[nm]);
	}
	fclose(f);
	return(0);
errOpen: fprintf(stderr,"\nwriteSPD() ERR '%s' read-only.",bb);return(1);
}

void writeSPDparHeader(FILE *fo, char *line){
	int ich,nch;
	strcpy(bprn,line);
	fprintf(fo,"#ln   ;    fname  ;score;uW/cm2; CIEx; CIEy;   CCT;Duv_off; Ra ;  Re; "
		"Scone; Mcone; Lcone; r-ELR; m-ELR;"
		"DuvTarg;   lx; m-EDI; m-EDI/W;  r/m;lx/W;"
		"Cost($);power(W);comfort; useful;");
	nch=atol(strtok(bprn,","));
	for(ich=0;ich<nch;ich++)fprintf(fo,"%4s;",strtok(NULL,","));
	fprintf(fo,"Ti(hr);");

}
int writeSPDpar(FILE *fo,int iline,char *fname,struct SPDparameters p, struct SPD_weight w, int nch){
	int k;
	for(k=0;k<2;k++)if(p.cri[k]<0.0)p.cri[k]=0.0;
	fprintf(fo,"\n%6d;%-11s;%6.0f;%6.0f;%.3lf;%.3lf;%6d;%7.4lf;%4.1lf;%4.1lf",
		iline,fname,p.score,p.irrad,p.ciexy[X],p.ciexy[Y],(int) p.cct, p.duv[DUV_DELTA], p.cri[0],p.cri[1]);	
	for(k=0;k<MALPHA-1;k++)fprintf(fo,";%6.3f",p.a_opic[k]);
	fprintf(fo,";%7.4lf",p.duv[DUV_TARGET]);
	if(p.a_opic[_PLUX]<9999.0)fprintf(fo,";%6.2f;%6.2f;",p.a_opic[_PLUX],p.mEDI);
	else fprintf(fo,";%.5e;%.5e;",p.a_opic[_PLUX],p.mEDI);
	fprintf(fo,"%.3lf;",p.mEDIperW);
	if(p.irrad>EPS)fprintf(fo,"%.3lf;",p.a_opic[_R]/p.a_opic[_M]); else fprintf(fo,"%.2lf;",-1.0);
	fprintf(fo,"%.3lf;",p.lxperW);
	for(k=0;k<2;k++)fprintf(fo,"%8.3lf;",p.cost[k]);
	for(k=0;k<2;k++)fprintf(fo,"%7.2lf;",p.user[k]);	
	for(k=0;k<nch;k++)fprintf(fo,"%4.2f;",w.fraction[k]);
	fprintf(fo,"%6.3lf",atof(fname)/3600);

	return(1);
}
#define VS_ReadSPDpar 241030
struct SPDparameters readSPDpar(FILE *f,struct SPDparameters p, struct SPD_weight *w, int nch){
	int iline,k;
	char fname[MB];
	double rm;
	if(fgets(bb,MB,f)==NULL){p.score=REJECT;return(p);} // EOF
	iline=atol(strtok(bb,";"));
	strcpy(fname,strtok(NULL,";"));
	p.score=atof(strtok(NULL,";"));
	p.irrad=atof(strtok(NULL,";"));
	p.ciexy[X]=atof(strtok(NULL,";"));
	p.ciexy[Y]=atof(strtok(NULL,";"));	
	p.cct=atof(strtok(NULL,";"));	
	p.duv[DUV_DELTA]=atof(strtok(NULL,";"));	
	p.cri[0]=atof(strtok(NULL,";"));	
	p.cri[1]=atof(strtok(NULL,";"));	
	for(k=0;k<MALPHA-1;k++)p.a_opic[k]=atof(strtok(NULL,";"));	
	p.duv[DUV_TARGET]=atof(strtok(NULL,";"));	
	p.a_opic[_PLUX]=atof(strtok(NULL,";"));
	p.mEDI=atof(strtok(NULL,";"));	
	p.mEDIperW=atof(strtok(NULL,";"));	
    rm=atof(strtok(NULL,";"));	
	p.lxperW=atof(strtok(NULL,";"));	
	for(k=0;k<2;k++)p.cost[k]=atof(strtok(NULL,";"));	
	for(k=0;k<2;k++)p.user[k]=atof(strtok(NULL,";"));	
	for(k=0;k<nch;k++)w->fraction[k]=atof(strtok(NULL,";"));
	return(p);
}		
	
	
	


